/*
 *      crypto-operations.c
 *
 *      Copyright 2006 Florian Sievers <florian@dopehouse.dyndns.org>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; version 2 of the License.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>

#include <gpgme.h>
#include <gtk/gtk.h>

#include "gpg-crypter.h"
#include "crypto-operations.h"
#include "error-dialogs.h"

#include "support.h"

/* encrypt the plaintext file */
void encrypt_file(GtkWidget *widget, gpointer data)
{
	gpgme_data_t dh_plain = 0, dh_cipher = 0;
	gpgme_error_t err;
	struct stat filestats;
	int plaintext_fd = -1, cipher_fd = -1;
	application_data *app_data;

	app_data = (application_data *) data;

	gpgme_set_armor(app_data->CTX, app_data->ASCIImode);
	// open plaintext file for reading
	plaintext_fd = open(app_data->plaintext_file->str, O_RDONLY);
	if(plaintext_fd == -1)
	{
		error_dialog_with_errno(errno);
		return;
	}
	/* get file attributes */
	stat(app_data->plaintext_file->str, &filestats);

	// try open cipher file for writing, error if exists
	cipher_fd = open(app_data->cipher_file->str, O_WRONLY | O_CREAT | O_EXCL, 600);
	if(cipher_fd == -1)
	{
		switch(errno)
		{
			case EEXIST: // ask to overwrite file if allready exists
				if(warn_dialog_with_errno(errno, _("Are you sure to overwrite the file?")) != 0)
				{
					return;
				}

				// try open cipher file for writing, overwride if exists
				cipher_fd = open(app_data->cipher_file->str, O_WRONLY | O_CREAT | O_TRUNC, 600);
				if(cipher_fd == -1)
				{
					error_dialog_with_errno(errno);
					return;
				}
				break;
			default:
				error_dialog_with_errno(errno);
				return;
				break;
		}
	}


	/* Create the data buffers */
	err = gpgme_data_new_from_fd(&dh_plain, plaintext_fd);
	if(err)
	{
		printf("3: %s\n", gpgme_strerror(err));
	}
	err = gpgme_data_new_from_fd(&dh_cipher, cipher_fd);
	if(err)
	{
		printf("3: %s\n", gpgme_strerror(err));
	}

	/* Encrypt plaintext */
	gpgme_op_encrypt(app_data->CTX, app_data->key, GPGME_ENCRYPT_ALWAYS_TRUST, dh_plain, dh_cipher);
	if(err)
	{
		printf("5: %s\n", gpgme_strerror(err));
	}

	/* Release data buffers */
	gpgme_data_release(dh_plain);
	gpgme_data_release(dh_cipher);
	close(cipher_fd);
	close(plaintext_fd);
	/* set same attributes for outputfile as inputfile */
	chmod(app_data->cipher_file->str, filestats.st_mode);

	/* End of program */
	return;
}

/* decrypt the cipher file */
void decrypt_file(GtkWidget *widget, gpointer data)
{
	gpgme_data_t dh_plain = 0, dh_cipher = 0;
	gpgme_error_t err;
	struct stat filestats;
	int plaintext_fd = -1, cipher_fd = -1;
	application_data *app_data;

	app_data = (application_data *) data;

	// open cipher file for reading
	cipher_fd = open(app_data->cipher_file->str, O_RDONLY);
	if(cipher_fd == -1)
	{
		error_dialog_with_errno(errno);
		return;
	}
	/* get file attributes */
	stat(app_data->cipher_file->str, &filestats);

	// try open plaintext file for writing, error if exists
	plaintext_fd = open(app_data->plaintext_file->str, O_WRONLY | O_CREAT | O_EXCL, 0600);
	if(plaintext_fd == -1)
	{
		switch(errno)
		{
			case EEXIST: // ask to overwrite file if allready exists
				if(warn_dialog_with_errno(errno, _("Are you sure to overwrite the file?")) != 0)
				{
					return;
				}

				// try open cipher file for writing, overwride if exists
				plaintext_fd = open(app_data->plaintext_file->str, O_WRONLY | O_CREAT | O_TRUNC, 0600);
				if(plaintext_fd == -1)
				{
					error_dialog_with_errno(errno);
					return;
				}
				break;
			default:
				error_dialog_with_errno(errno);
				return;
				break;
		}
	}

	/* Create the data buffers */
	err = gpgme_data_new_from_fd(&dh_cipher, cipher_fd);
	if(err)
	{
		printf("3: %s\n", gpgme_strerror(err));
	}
	err = gpgme_data_new_from_fd(&dh_plain, plaintext_fd);
	if(err)
	{
		printf("3: %s\n", gpgme_strerror(err));
	}

	/* decrypt cipher */
	err = gpgme_op_decrypt(app_data->CTX, dh_cipher, dh_plain);
	if(err != 0)
	{
		GtkWidget *error_dialog;
		error_dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, "%s: %s", g_locale_to_utf8(gpgme_strsource(err), -1, NULL, NULL, NULL), g_locale_to_utf8(gpgme_strerror(err), -1, NULL, NULL, NULL));
		gtk_dialog_run(GTK_DIALOG(error_dialog));
		gtk_widget_destroy(GTK_WIDGET(error_dialog));
	}

	/* Release data buffers */
	gpgme_data_release(dh_plain);
	gpgme_data_release(dh_cipher);
	close(plaintext_fd);
	close(cipher_fd);
	/* set same attributes for outputfile as inputfile */
	chmod(app_data->plaintext_file->str, filestats.st_mode);

	/* End of program */
	return;
}

/* encrypt the plaintext buffer */
void encrypt_buffer(GtkWidget *widget, gpointer data)
{
	gpgme_data_t dh_plain = 0, dh_cipher = 0;
	gpgme_error_t err;
	size_t nRead = 0;
	char *plaintext;
	gint buffer_length;
	gchar *cipher = 0;
	GtkTextIter iter_start, iter_end;
	application_data *app_data;

	app_data = (application_data *) data;

	gpgme_set_armor(app_data->CTX, 1);

	gtk_text_buffer_get_start_iter(GTK_TEXT_BUFFER(app_data->buffer_plaintext), &iter_start);
	gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(app_data->buffer_plaintext), &iter_end);
	plaintext = gtk_text_buffer_get_text(GTK_TEXT_BUFFER(app_data->buffer_plaintext), &iter_start, &iter_end, FALSE);

	/* Create the data buffers */
	err = gpgme_data_new_from_mem(&dh_plain, plaintext, strlen(plaintext), 0);
	if(err)
	{
		printf("3: %s\n", gpgme_strerror(err));
	}
	err = gpgme_data_new(&dh_cipher);
	if(err)
	{
		printf("4: %s\n", gpgme_strerror(err));
	}

	/* Encrypt plaintext */
	err = gpgme_op_encrypt(app_data->CTX, app_data->key, GPGME_ENCRYPT_ALWAYS_TRUST, dh_plain, dh_cipher);
	if(err)
	{
		printf("5: %s\n", gpgme_strerror(err));
	}

	/* Output ascii amored cipher */
	cipher = gpgme_data_release_and_get_mem(dh_cipher, &nRead);
	dh_cipher = 0;
	cipher = (char *)realloc(cipher, nRead + 1);
	cipher[nRead] = 0;
	buffer_length = strlen(cipher);
	gtk_text_buffer_set_text(GTK_TEXT_BUFFER(app_data->buffer_cipher), cipher, buffer_length);
	gpgme_free(cipher);

	/* Release data buffers */
	gpgme_data_release(dh_plain);
	gpgme_data_release(dh_cipher);

	/* End of program */
	return;
}

/* decrypt the cipher buffer */
void decrypt_buffer(GtkWidget *widget, gpointer data)
{
	gpgme_data_t dh_plain = 0, dh_cipher = 0;
	gpgme_error_t err;
	size_t nRead = 0;
	char *cipher;
	gchar *plaintext = 0;
	gint buffer_length;
	GtkTextIter iter_start, iter_end;
	application_data *app_data;

	app_data = (application_data *) data;

	//buffer_length = gtk_text_buffer_get_char_count(GTK_TEXT_BUFFER(app_data->buffer_cipher));
	gtk_text_buffer_get_start_iter(GTK_TEXT_BUFFER(app_data->buffer_cipher), &iter_start);
	gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(app_data->buffer_cipher), &iter_end);
	cipher = gtk_text_buffer_get_text(GTK_TEXT_BUFFER(app_data->buffer_cipher), &iter_start, &iter_end, FALSE);


	/* Create the data buffers */
	err = gpgme_data_new_from_mem(&dh_cipher, cipher, strlen(cipher), 0);
	if(err)
	{
		printf("3: %s\n", gpgme_strerror(err));
	}
	err = gpgme_data_new(&dh_plain);
	if(err)
	{
		printf("4: %s\n", gpgme_strerror(err));
	}

	/* decrypt cipher */
	err = gpgme_op_decrypt(app_data->CTX, dh_cipher, dh_plain);
	if(err != 0)
	{
		GtkWidget *error_dialog;
		error_dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, "%s: %s", g_locale_to_utf8(gpgme_strsource(err), -1, NULL, NULL, NULL), g_locale_to_utf8(gpgme_strerror(err), -1, NULL, NULL, NULL));
		gtk_dialog_run(GTK_DIALOG(error_dialog));
		gtk_widget_destroy(GTK_WIDGET(error_dialog));
	}
	else
	{
		/* Output ascii amored cipher */
		plaintext = gpgme_data_release_and_get_mem(dh_plain, &nRead);
		dh_plain = 0;
		plaintext = (char *)realloc(plaintext, nRead + 1);
		plaintext[nRead] = 0;
		buffer_length = strlen(plaintext);

		/* check if the plaintext is utf-8 or not */
		if(g_utf8_validate(plaintext, -1, NULL))
		{
			/* if utf-8 just copy to text buffer */
			gtk_text_buffer_set_text(GTK_TEXT_BUFFER(app_data->buffer_plaintext), plaintext, buffer_length);
		}
		else
		{
			/* if not utf-8 convert to utf-8 and copy to text buffer */
			gtk_text_buffer_set_text(GTK_TEXT_BUFFER(app_data->buffer_plaintext), g_locale_to_utf8(plaintext, -1, NULL, NULL, NULL), buffer_length);
		}
		gpgme_free(cipher);
	}

	/* Release data buffers */
	gpgme_data_release(dh_plain);
	gpgme_data_release(dh_cipher);

	/* End of program */
	return;
}
