/*
 *      file-operations.c
 *
 *      Copyright 2007 Florian Sievers <florian@dopehouse.dyndns.org>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#ifdef HAVE_LOCALE_H
	#include <locale.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>

#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gstdio.h>

#include "file-operations.h"
#include "error-dialogs.h"

#include "support.h"

gboolean save_operation(GtkWidget *widget, gchar *filename, gboolean overwrite);


gchar *open_file(GtkWidget *widget, gchar *filename)
{
	GtkWidget *file_chooser;
	GtkTextBuffer *text_buffer;
	gchar *text_from_file = NULL;
	gboolean ret = 0;
	GError *error = NULL;

	/* free filename if not NULL */
	if(filename != NULL)
	{
		g_free(filename);
		filename = NULL;
	}

	/* create and run file chooser dialog */
	file_chooser = gtk_file_chooser_dialog_new(_("Open file"), NULL, GTK_FILE_CHOOSER_ACTION_OPEN, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
	gtk_file_chooser_set_local_only(GTK_FILE_CHOOSER(file_chooser), TRUE);
	gtk_dialog_set_default_response(GTK_DIALOG(file_chooser), GTK_RESPONSE_ACCEPT);

	if(gtk_dialog_run(GTK_DIALOG(file_chooser)) == GTK_RESPONSE_ACCEPT)
	{
		filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(file_chooser));

		gtk_widget_destroy(file_chooser);

		/* get text buffer */
		text_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));

		/* write text to file */
		ret = g_file_get_contents(filename, &text_from_file, NULL, &error);

		if(ret != FALSE)
		{
			/* store text from file into text buffer */
			gtk_text_buffer_set_text(GTK_TEXT_BUFFER(text_buffer), text_from_file, -1);

			// free leettext
			g_free(text_from_file);

			return filename;
		}

		g_free(filename);
		return NULL;
	}
	else
	{
		gtk_widget_destroy(file_chooser);
	}

	g_free(filename);
	return NULL;
}


gchar *save_file(GtkWidget *widget, gchar *filename)
{
	if(filename == NULL)
	{
		return save_as_file(widget, filename, TRUE);
	}

	if(save_operation(widget, filename, FALSE) != TRUE)
	{
		return NULL;
	}
	return filename;
}


gchar *save_as_file(GtkWidget *widget, gchar *filename, gboolean ask_overwrite)
{
	// if ask_overwrite == FALSE --> don't warn on overwrite
	GtkWidget *file_chooser;

	/* free filename if not NULL */
	if(filename != NULL)
	{
		g_free(filename);
		filename = NULL;
	}

	/* create and run file chooser dialog */
	file_chooser = gtk_file_chooser_dialog_new(_("Save file"), NULL, GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
	gtk_file_chooser_set_local_only(GTK_FILE_CHOOSER(file_chooser), TRUE);
	gtk_dialog_set_default_response(GTK_DIALOG(file_chooser), GTK_RESPONSE_ACCEPT);

	if(gtk_dialog_run(GTK_DIALOG(file_chooser)) == GTK_RESPONSE_ACCEPT)
	{
		filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(file_chooser));

		gtk_widget_destroy(file_chooser);

		if(save_operation(widget, filename, ask_overwrite) != TRUE)
		{
			return NULL;
		}
		return filename;
	}
	else
	{
		gtk_widget_destroy(file_chooser);
	}

	g_free(filename);
	return NULL;
}


gboolean save_operation(GtkWidget *widget, gchar *filename, gboolean ask_overwrite)
{
	// if ask_overwrite == TRUE --> don't warn on overwrite
	GtkTextBuffer *text_buffer;
	GtkTextIter iter_start, iter_end;
	gchar *text_from_buffer = NULL;
	gboolean ret = 0;
	GError *error = NULL;

	/* get text buffer */
	text_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
	gtk_text_buffer_get_start_iter(GTK_TEXT_BUFFER(text_buffer), &iter_start);
	gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(text_buffer), &iter_end);
	text_from_buffer = gtk_text_buffer_get_text(GTK_TEXT_BUFFER(text_buffer), &iter_start, &iter_end, FALSE);

	/* warn if file allready exist */
	if((g_file_test(filename, G_FILE_TEST_EXISTS)) == TRUE && ask_overwrite == TRUE)
	{
			if(warn_dialog_with_string(_("File already exist."), _("Overwrite this file?")) != 0)
			{
				g_free(filename);
				return FALSE;
			}
	}
	/* write text to file */
	ret = g_file_set_contents(filename, text_from_buffer, -1, &error);

	// free leettext
	g_free(text_from_buffer);

	if(ret != TRUE)
	{
		g_free(filename);
		return FALSE;
	}
	return TRUE;
}

